# Schema for holding inventory of applications of interest (AOI).

# aoi table stores master list of facility aoi's. 

# Columns:
#   aoi_id 			- primary key
#   aoi_name		-
#   aoi_cognizant1_id	-
#   aoi_cognizant2_id	- 
#   aoi_customer_contact_id	-
#   aoi_status_id		-
#   aoi_description		-
#   aoi_func_criteria	-
#   aoi_keyword		-


drop table if exists aoi;
create table aoi
  (
    aoi_id int not null primary key auto_increment,
    aoi_name varchar(60),
    aoi_cognizant1_id int,
    aoi_cognizant2_id int,
    aoi_customer_contact_id int,
    aoi_status_id int,
    aoi_description varchar(255),
    aoi_func_criteria varchar(255),
    aoi_keyword varchar(255),
    unique(aoi_name)
  ) type=InnoDb;

drop table if exists aoi_machine;
create table aoi_machine
  (
    aoi_machine_id int not null primary key auto_increment,
    machine_id int,
    aoi_id int
  ) type=InnoDb;

drop table if exists aoi_techsys;
create table aoi_techsys
  (
    aoi_techsystem_id int not null primary key auto_increment,
    technical_system_id int,
    aoi_id int
  ) type=InnoDb;

drop table if exists aoi_ioc;
create table aoi_ioc
  (
    aoi_ioc_id int not null primary key auto_increment,
    ioc_id int,
    aoi_id int
  ) type=InnoDb;

drop table if exists aoi_plc;
create table aoi_plc
  (
    aoi_plc_id int not null primary key auto_increment,
    plc_id int,
    aoi_id int
  ) type=InnoDb;

drop table if exists plc;
create table plc
  (
    plc_id int not null primary key auto_increment,
    plc_name varchar(60),
    plc_programmer int,
    plc_source_code_directory varchar(255),
    plc_source_code_pdf_path varchar(255),
    component_id int,
    unique(plc_name)
  ) type=InnoDb;

drop table if exists aoi_status;
create table aoi_status
  (
    aoi_status_id int not null primary key auto_increment,
    aoi_status varchar(40)
  ) type=InnoDb;

drop table if exists aoi_document;
create table aoi_document
  (
    doc_id int not null primary key auto_increment,
    aoi_id int,
    uri_id int,
    doc_type_id int
  ) type=InnoDb;

drop table if exists doc_type;
create table doc_type
  (
    doc_type_id int not null primary key auto_increment,
    doc_type varchar(40),
    unique(doc_type)
  ) type=InnoDb;

drop table if exists aoi_topdisplay;
create table aoi_topdisplay
  (
    aoi_topdisplay_id int not null primary key auto_increment,
    aoi_id int,
    uri_id int
  ) type=InnoDb;

drop table if exists aoi_note;
create table aoi_note
  (
    aoi_note_id int not null primary key auto_increment,
    aoi_id int,
    aoi_note_date timestamp(14),
    aoi_note varchar(255)
  ) type=InnoDb;

drop table if exists aoi_revhistory;
create table aoi_revhistory
  (
    aoi_revhistory_id int not null primary key auto_increment,
    aoi_id int,
    aoi_revhistory_date timestamp(14),
    aoi_revhistory_comment varchar(255)
  ) type=InnoDb;

drop table if exists aoi_relation;
create table aoi_relation
  (
    aoi_relation_id int not null primary key auto_increment,
    aoi1_id int,
    aoi2_id int,
    aoi1_relation_type_id int,
    aoi2_relation_type_id int
  ) type=InnoDb;

drop table if exists aoi_relation_type;
create table aoi_relation_type
  (
    aoi_relation_type_id int not null primary key auto_increment,
    aoi_relation varchar(40),
    unique(aoi_relation)
  ) type=InnoDb;

drop table if exists aoi_parent_child;
create table aoi_parent_child
  (
    aoi_parent_child_id int not null primary key auto_increment,
    aoi_parent_id int,
    aoi_child_id int
  ) type=InnoDb;

drop table if exists aoi_criticality;
create table aoi_criticality
  (
    aoi_criticality_id int not null primary key auto_increment,
    aoi_id int,
    criticality_id int
  ) type=InnoDb;

drop table if exists criticality_type;
create table criticality_type
  (
    criticality_id int not null primary key auto_increment,
    criticality varchar(100),
    description varchar(255),
    unique(criticality)
  ) type=InnoDb;

drop table if exists aoi_dbload;
create table aoi_dbload
  (
    aoi_dbload_id int not null primary key auto_increment,
    aoi_id int,
    uri_static_id int
  ) type=InnoDb;

drop table if exists uri_static;
create table uri_static
  (
    uri_static_id int not null primary key auto_increment,
    uri varchar(255),
    subst_str varchar(255),
    unique(uri)
  ) type=InnoDb;

drop table if exists aoi_rec;
create table aoi_rec
  (
    aoi_rec_id int not null primary key auto_increment,
    rec_static_id int,
    aoi_id int
  ) type=InnoDb;

drop table if exists rec_static;
create table rec_static
  (
    rec_static_id int not null primary key auto_increment,
    rec_name varchar(128),
    rec_type_id int,
    ioc_boot_id int,
    unique(rec_name)
  ) type=InnoDb;

