package NetworkCrawlerParser;

# This package provides routines for writing ioc network interface data
# to a MySQL database by parsing a nightly updated network data file. 

use DBI qw(:sql_types);
require Exporter;

use Mail::Mailer;
use File::stat;
use Cwd;

use LogUtil;

our @ISA = qw(Exporter);
our @EXPORT = qw( $email_body &send_email &get_modified_network_data 
    &get_all_network_data);
our @EXPORT_OK = qw();
our $VERSION = 1.00;

$email_body="";

#=============================================================================
# Public
sub get_modified_network_data {
  my $infileDiffSorted = $_[0];
  my $network_hits_ref = $_[1];

  my $ioc_id;
  my $ioc_row;
  my $select_line;
  my $line;
  my $iocEtypeHold = '';

  if ( ! open(INFILE,$infileDiffSorted) ) {
    LogUtil::log('error',"Cannot open sorted diff file $infileDiffSorted: $@");   
    $email_body .= "ERROR: Cannot open diff file $infileDiffSorted: $@\n";
    return -1;
  }

  while($line = <INFILE>)
  {
    LogUtil::log('debug',"Input line: $line");   
    $line =~ s/\015?\012//; # remove CR LF  characters

    ($updateType,$switch,$bladePort,$iocEtype,$connectStatus) = split(/\s+/,$line);

    # Use only network data for iocs
    if ( !defined $switch || $switch =~ m/^Switch/ ||
         !defined $connectStatus || !defined $iocEtype ||
         $iocEtype eq "connected" || $iocEtype eq "notconnect") {
      next;
    }

    # Get ioc name from input file
    # Cant use split because ioc may have embedded dashes
    next if substr($iocEtype,length($iocEtype)-2,1) ne "-";
    $ioc = substr($iocEtype,0,length($iocEtype)-2);
    $Etype = substr($iocEtype,length($iocEtype)-1,1);

    # Use only network data for iocs primary/secondary interfaces
    if ( $ioc =~ m/^hub/  || !defined $Etype || ( $Etype !~ m/s/ &&  $Etype !~ m/p/) )  {
      LogUtil::log('debug',"$line");   
      next;
    }

    if ($iocEtypeHold ne '' && $iocEtype ne $iocEtypeHold ) { 
      push(@network_hits_ref,[$iocHold,$EtypeHold,'','',0]);   
      LogUtil::log('debug',"Modified data: $iocHold,$EtypeHold,'','',0");   
    }

    # May have to add ioc prefix
    if ( $ioc !~ m/^ioc/ ) {
      LogUtil::log('warn',"Prefix `ioc` was added:  $ioc");   
      $email_body .= "Prefix 'ioc' was added: $ioc.\n";
      $ioc =~ s/^/ioc/ 
    }

    if ($updateType eq "<") {
      $iocEtypeHold = $iocEtype;
      $iocHold = $ioc;
      $EtypeHold = $Etype;
      $switchHold = $switch;
      $bladePortHold = $bladePort;
    } else {
      if ( $iocEtype eq $iocEtypeHold  && ( $switchHold ne $switch 
           || $bladePortHold ne $bladePort ) ) { 
        ($blade,$port) = split(/\//,$bladePort);
        push(@network_hits_ref,[$ioc,$Etype,$switch,$blade,$port]);   
        LogUtil::log('debug',"Modified data: $ioc,$Etype,$switch,$blade,$port");   
      }
      $iocEtypeHold = '';
    }
  }
  close(INFILE);

  return 0;
}

#=============================================================================
# Public
sub get_all_network_data {
  my $infile = $_[0];
  my $network_hits_ref = $_[1];

  LogUtil::log('info',"Updating all ioc network data.");   
  LogUtil::log('info',"New accel_sw.log file date: $infile_date");   
  $email_body .= "Updating all ioc network data.\n";
  $email_body .= "New accel_sw.log file date: $infile_date\n\n";

  if ( ! open(INFILE,$infile) ) {
    LogUtil::log('error',"Cannot open $infile: $@");   
    $email_body .= "ERROR: Cannot open $infile: $@\n";
    return -1;
  }

  my $line;

  while($line = <INFILE>)
  {
    $line =~ s/\015?\012//; # remove CR LF  characters

    ($switch,$bladePort,$iocEtype,$connectStatus) = split(/\s+/,$line);

    if ( !defined $switch || $switch =~ m/^Switch/ ||
         !defined $connectStatus || $bladePort =~ m/^---/  ||
         $iocEtype eq "connected" || $iocEtype eq "notconnect") {
         LogUtil::log('debug',"$line");   
        next;
    }

    ($blade,$port) = split(/\//,$bladePort);

    # Get ioc name from input file
    # Cant use split because ioc may have embedded dashes
    next if substr($iocEtype,length($iocEtype)-2,1) ne "-";
    $ioc = substr($iocEtype,0,length($iocEtype)-2);
    $Etype = substr($iocEtype,length($iocEtype)-1,1);

    if ( $ioc =~ m/^hub/  || !defined $Etype || ( $Etype !~ m/s/ &&  $Etype !~ m/p/) )  {
        LogUtil::log('debug',"$line");   
        next;
    }

    # May have to add ioc prefix
    if ( $ioc !~ m/^ioc/ ) {
      LogUtil::log('warn',"Prefix `ioc` was added:  $ioc");   
      $email_body .= "Prefix 'ioc' was added: $ioc.\n";
      $ioc =~ s/^/ioc/ 
    }

    push(@network_hits_ref,[$ioc,$Etype,$switch,$blade,$port]);   
  }
  close(INFILE);

  return 0;
}

#=============================================================================
# Public
sub send_email {
  my $body = $_[0]."\n";
  my $from_address = "updateIocNetworkData";
  my $to_address = "jba";
  my $subject = "ioc network data updates";

  $mailer = Mail::Mailer->new();
  $mailer->open({ From	=> $from_address,
                To	=> $to_address,
                Subject	=> $subject,
              }) or die "Can't open: $! \n";

  print $mailer $body;
  $mailer->close();
}

1;

